@@ -1,6 +1,6 @@
 /* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
- * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -46,16 +46,16 @@ static TumblerThumbnail *xdg_cache_cache_get_thumbnail      (TumblerCache
                                                              const gchar            *uri,
                                                              TumblerThumbnailFlavor *flavor);
 static void              xdg_cache_cache_cleanup            (TumblerCache           *cache,
-                                                             const gchar            *uri_prefix,
+                                                             const gchar *const     *base_uris,
                                                              guint64                 since);
 static void              xdg_cache_cache_delete             (TumblerCache           *cache,
-                                                             const GStrv             uris);
+                                                             const gchar *const     *uris);
 static void              xdg_cache_cache_copy               (TumblerCache           *cache,
-                                                             const GStrv             from_uris,
-                                                             const GStrv             to_uris);
+                                                             const gchar *const     *from_uris,
+                                                             const gchar *const     *to_uris);
 static void              xdg_cache_cache_move               (TumblerCache           *cache,
-                                                             const GStrv             from_uris,
-                                                             const GStrv             to_uris);
+                                                             const gchar *const     *from_uris,
+                                                             const gchar *const     *to_uris);
 static gboolean          xdg_cache_cache_is_thumbnail       (TumblerCache           *cache,
                                                              const gchar            *uri);
 static GList            *xdg_cache_cache_get_flavors        (TumblerCache           *cache);
@@ -169,55 +169,97 @@ xdg_cache_cache_get_thumbnail (TumblerCache           *cache,
 
 
 static void
-xdg_cache_cache_cleanup (TumblerCache *cache,
-                         const gchar  *uri_prefix,
-                         guint64       since)
+xdg_cache_cache_cleanup (TumblerCache       *cache,
+                         const gchar *const *base_uris,
+                         guint64             since)
 {
   XDGCacheCache *xdg_cache = XDG_CACHE_CACHE (cache);
   const gchar   *file_basename;
   guint64        mtime;
+  GFile         *base_file;
   GFile         *dummy_file;
+  GFile         *original_file;
   GFile         *parent;
   GList         *iter;
   gchar         *dirname;
   gchar         *filename;
   gchar         *uri;
+  guint          n;
   GDir          *dir;
 
   g_return_if_fail (XDG_CACHE_IS_CACHE (cache));
-  
+
+  /* iterate over all flavors */
   for (iter = xdg_cache->flavors; iter != NULL; iter = iter->next)
     {
+      /* compute the flavor directory filename */
       dummy_file = xdg_cache_cache_get_file ("foo", iter->data);
       parent = g_file_get_parent (dummy_file);
       dirname = g_file_get_path (parent);
       g_object_unref (parent);
       g_object_unref (dummy_file);
 
+      /* attempt to open the directory for reading */
       dir = g_dir_open (dirname, 0, NULL);
-
       if (dir != NULL)
         {
-          while ((file_basename = g_dir_read_name (dir)) != NULL)
+          /* iterate over all files in the directory */
+          file_basename = g_dir_read_name (dir);
+          while (file_basename != NULL)
             {
+              /* build the thumbnail filename */
               filename = g_build_filename (dirname, file_basename, NULL);
 
+              /* read thumbnail information from the file */
               if (xdg_cache_cache_read_thumbnail_info (filename, &uri, &mtime, 
                                                        NULL, NULL))
                 {
-                  if ((uri_prefix == NULL || uri == NULL) 
-                      || (g_str_has_prefix (uri, uri_prefix) && (mtime <= since)))
+                  /* check if the thumbnail information is valid or the mtime
+                   * is too old */
+                  if (uri == NULL || mtime <= since) 
                     {
+                      /* it's invalid, so let's remove the thumbnail */
                       g_unlink (filename);
                     }
+                  else
+                    {
+                      /* create a GFile for the original URI. we need this for
+                       * reliably checking the ancestor/descendant relationship */
+                      original_file = g_file_new_for_uri (uri);
+
+                      for (n = 0; base_uris != NULL && base_uris[n] != NULL; ++n)
+                        {
+                          /* create a GFile for the base URI */
+                          base_file = g_file_new_for_uri (base_uris[n]);
+
+                          /* delete the file if it is a descendant of the base URI */
+                          if (g_file_equal (original_file, base_file)
+                              || g_file_has_prefix (original_file, base_file))
+                            {
+                              g_unlink (filename);
+                            }
+
+                          /* releas the base file */
+                          g_object_unref(base_file);
+                        }
+
+                      /* release the original file */
+                      g_object_unref (original_file);
+                    }
                 }
 
+              /* free the thumbnail filename */
               g_free (filename);
+
+              /* try to determine the next filename in the directory */
+              file_basename = g_dir_read_name (dir);
             }
 
+          /* close the handle used to reading from the directory */
           g_dir_close (dir);
         }
 
+      /* free the thumbnail flavor directory filename */
       g_free (dirname);
     }
 }
@@ -225,8 +267,8 @@ xdg_cache_cache_cleanup (TumblerCache *cache,
 
 
 static void
-xdg_cache_cache_delete (TumblerCache *cache,
-                        const GStrv   uris)
+xdg_cache_cache_delete (TumblerCache       *cache,
+                        const gchar *const *uris)
 {
   XDGCacheCache *xdg_cache = XDG_CACHE_CACHE (cache);
   GList         *iter;
@@ -250,9 +292,9 @@ xdg_cache_cache_delete (TumblerCache *cache,
 
 
 static void
-xdg_cache_cache_copy (TumblerCache *cache,
-                      const GStrv   from_uris,
-                      const GStrv   to_uris)
+xdg_cache_cache_copy (TumblerCache       *cache,
+                      const gchar *const *from_uris,
+                      const gchar *const *to_uris)
 {
   XDGCacheCache *xdg_cache = XDG_CACHE_CACHE (cache);
   GFileInfo     *info;
@@ -269,11 +311,10 @@ xdg_cache_cache_copy (TumblerCache *cache,
   g_return_if_fail (XDG_CACHE_IS_CACHE (cache));
   g_return_if_fail (from_uris != NULL);
   g_return_if_fail (to_uris != NULL);
-  g_return_if_fail (g_strv_length (from_uris) == g_strv_length (to_uris));
 
   for (iter = xdg_cache->flavors; iter != NULL; iter = iter->next)
     {
-      for (n = 0; n < g_strv_length (from_uris); ++n)
+      for (n = 0; n < g_strv_length ((gchar **)from_uris); ++n)
         {
           dest_source_file = g_file_new_for_uri (to_uris[n]);
           info = g_file_query_info (dest_source_file, G_FILE_ATTRIBUTE_TIME_MODIFIED,
@@ -325,9 +366,9 @@ xdg_cache_cache_copy (TumblerCache *cache,
 
 
 static void
-xdg_cache_cache_move (TumblerCache *cache,
-                      const GStrv   from_uris,
-                      const GStrv   to_uris)
+xdg_cache_cache_move (TumblerCache       *cache,
+                      const gchar *const *from_uris,
+                      const gchar *const *to_uris)
 {
   XDGCacheCache *xdg_cache = XDG_CACHE_CACHE (cache);
   GFileInfo     *info;
@@ -345,11 +386,10 @@ xdg_cache_cache_move (TumblerCache *cache,
   g_return_if_fail (XDG_CACHE_IS_CACHE (cache));
   g_return_if_fail (from_uris != NULL);
   g_return_if_fail (to_uris != NULL);
-  g_return_if_fail (g_strv_length (from_uris) == g_strv_length (to_uris));
 
   for (iter = xdg_cache->flavors; iter != NULL; iter = iter->next)
     {
-      for (n = 0; n < g_strv_length (from_uris); ++n)
+      for (n = 0; n < g_strv_length ((gchar **)from_uris); ++n)
         {
           dest_source_file = g_file_new_for_uri (to_uris[n]);
           info = g_file_query_info (dest_source_file, G_FILE_ATTRIBUTE_TIME_MODIFIED,
@@ -621,7 +661,7 @@ xdg_cache_cache_read_thumbnail_info (const gchar  *filename,
 
 gboolean
 xdg_cache_cache_write_thumbnail_info (const gchar  *filename,
-                                      gchar        *uri,
+                                      const gchar  *uri,
                                       guint64       mtime,
                                       GCancellable *cancellable,
                                       GError      **error)
